<footer>
  {% if next_page or previous_page %}
    <div class="rst-footer-buttons" role="navigation" aria-label="footer navigation">
      {% if next_page %}
        <a href="{{ next_page.url }}" class="btn btn-neutral float-right" title="{{ next_page.title }}"/>Next <span class="icon icon-circle-arrow-right"></span></a>
      {% endif %}
      {% if previous_page %}
        <a href="{{ previous_page.url }}" class="btn btn-neutral" title="{{ previous_page.title }}"><span class="icon icon-circle-arrow-left"></span> Previous</a>
      {% endif %}
    </div>
  {% endif %}

  <hr/>

  <div role="contentinfo">
    <p>
    <!-- Copyright etc -->
    </p>
  </div>

  <a href="http://https://github.com/stevengj/mpb">
    <span class="SmallCaps">mpb</span> </a> documentation

  <p>
  <font size=-1>
  Built with <a href="http://www.mkdocs.org">MkDocs</a> 
  using a customized theme modeled 
  after the 
  <a href="https://github.com/snide/sphinx_rtd_theme">theme</a>
  provided by <a href="https://readthedocs.org">Read the Docs</a>.
  </font>

</footer>
